; -------------------------------------------------------------------------------------
; FUNCOES DE CONTROLE PARA LCD GRAFICO KS0108
;
; Incluir este arquivo no inicio do codigo fonte.
;
; -------------------------------------------------------------------------------------

; -------------------------------------------------------------------------------------
; Incluir no programa os simbolos e registradores abaixo:
;
;#define GLCD_DATATRIS TRISB	  ; controle de I/O da porta de dados/comandos
;#define GLCD_DATAIN   PORTB      ; porta de dados/comandos
;#define GLCD_DATAOUT  LATB       ; alias da porta de dados/comandos

;#define GLCD_CNT      PORTC      ; porta de controle para o LCD
;#define GLCD_CNTRIS   TRISC      ; controle de I/O da porta de controle
;#define GLCD_CNTDI    0          ; ao pino DI  do LCD........ dado/ou instrucao  
;#define GLCD_CNTRW    1          ; ao pino RW  do LCD........ read/write 
;#define GLCD_CNTE     2          ; ao pino EN  do LCD........ chip enable 
;#define GLCD_CNTCS1   3          ; ao pino CS1 do LCD........ lado 1  
;#define GLCD_CNTCS2   4          ; ao pino CS2 do LCD........ lado 2  
;#define GLCD_CNTRST   5          ; ao pino RST do LCD........ restart

;GLCD_W1         RES     1         ; registrador de trabalho  
;GLCD_W2         RES     1         ; registrador de trabalho  
;GLCD_Delay1     RES     1         ; registrador de trabalho
;GLCD_RC1        RES     1         ; registrador de controle 
;GLCD_PAGE       RES     1         ; numero da pagina
;GLCD_COL        RES     1         ; numero da coluna
; -------------------------------------------------------------------------------------


#define X_ADDRESS    0xB8         ; Comando Set X Address 
#define Y_ADDRESS    0x40         ; Comando Set Y Address 
#define START_LINE   0xC0         ; Comando Start Line
#define DISPLAY_ON   0x3F         ; Comando Display on	  
#define DISPLAY_OFF  0x3E         ; Comando Display off	  
#define BUSY         0x80         ; mascara para o bit de status do LCD


; ------------------------------------------------------------------------------
; Inicializa o LCD
;
; ENTRADA: Nao ha
; ------------------------------------------------------------------------------
GLCD_Init   movlw   0x00                          ; W=0
            movwf   GLCD_DATATRIS                 ; porta de dados ---> saida
            movwf   GLCD_CNTRIS                   ; porta de controle ---> saida
            movwf   GLCD_DATAOUT                  ; limpa porta de dados
            clrf    GLCD_RC1                      ; limpa registrador de controle
            movf    GLCD_RC1,w                    ; carrega registrador de controle
            movwf   GLCD_CNT                      ; envia controles para porta
            nop                                   ; delay esperando estabilizar
            nop                                   ; delay esperando estabilizar
            nop                                   ; delay esperando estabilizar
            bcf     GLCD_RC1,GLCD_CNTRST          ; RST=0
            movf    GLCD_RC1,w                    ; carrega registrador de controle
            movwf   GLCD_CNT                      ; envia controles para porta
            movlw   .255                          ; tempo
            call    GLCD_Delay                    ; aguarda completar
            movlw   .255                          ; tempo
            call    GLCD_Delay                    ; aguarda completar
            bsf     GLCD_RC1,GLCD_CNTRST          ; RST=1
            movf    GLCD_RC1,w                    ; carrega registrador de controle
            movwf   GLCD_CNT                      ; envia controles para porta
            movlw   .255                          ; tempo
            call    GLCD_Delay                    ; aguarda completar
            movlw   .255                          ; tempo
            call    GLCD_Delay                    ; aguarda completar
            bsf     GLCD_RC1,GLCD_CNTCS1          ; habilita lado 1
            bsf     GLCD_RC1,GLCD_CNTCS2          ; habilita lado 2
            movf    GLCD_RC1,w                    ; carrega registrador de controle
            movwf   GLCD_CNT                      ; envia controles para porta
            nop                                   ; delay esperando estabilizar 
            nop                                   ; delay esperando estabilizar
            nop                                   ; delay esperando estabilizar
            movlw   DISPLAY_ON                    ; DISPLAY_ON 
            call    GLCD_Command                  ; envia comando ao LCD
            movlw   START_LINE                    ; START_LINE 
            call    GLCD_Command                  ; envia comando ao LCD 
            movlw   X_ADDRESS                     ; X_ADDRESS - posiciona na pagina 0  
            call    GLCD_Command                  ; envia comando ao LCD
            movlw   Y_ADDRESS                     ; Y_ADDRESS - posiciona na coluna 0
            call    GLCD_Command                  ; envia comando ao LCD
            bcf     GLCD_RC1,GLCD_CNTCS1          ; desabilita lado 1
            bcf     GLCD_RC1,GLCD_CNTCS2          ; desabilita lado 2
            movf    GLCD_RC1,w                    ; carrega registrador de controle
            movwf   GLCD_CNT                      ; envia controles para porta
            return                                ; fim



; ------------------------------------------------------------------------------
; Envia um comando ao LCD
;
; ENTRADA: W contem o codigo do comando
; ------------------------------------------------------------------------------
GLCD_Command
           movwf    GLCD_W1                       ; salva o comando
           call     GLCD_Wait                     ; aguarda o LCD ficar pronto
           bsf      GLCD_RC1,GLCD_CNTRST          ; assegura que RST=1
           bcf      GLCD_RC1,GLCD_CNTDI           ; indica comando
           bcf      GLCD_RC1,GLCD_CNTRW           ; indica gravacao
           bsf      GLCD_RC1,GLCD_CNTE            ; strobe up
           clrf     GLCD_CNTRIS                   ; porta de controle como saida
           movf     GLCD_RC1,w                    ; carrega registrador de controle
           movwf    GLCD_CNT                      ; output na porta de controle
           movlw    10                            ; tempo
           call     GLCD_Delay                    ; duty cycle
           movf     GLCD_W1,w                     ; restaura o comando
           movwf    GLCD_DATAOUT                  ; envia o comando
           nop                                    ; delay esperando estabilizar
           nop                                    ; delay esperando estabilizar
           nop                                    ; delay esperando estabilizar
           bcf      GLCD_RC1,GLCD_CNTE            ; strobe down
           movf     GLCD_RC1,w                    ; carrega registrador de controle
           movwf    GLCD_CNT                      ; output na porta de controle
           return                                 ; fim


; ----------------------------------------------------------------------------------------------------------
; Aguarda o LCD ficar pronto para operar
;
; ENTRADA: nao ha
;
; ----------------------------------------------------------------------------------------------------------
GLCD_Wait  movlw    0xFF                         ; mascara de I/O 
           movwf    GLCD_DATATRIS                ; porta de dados como entrada
           bcf      INTCON2,RBPU                 ; habilita resistores pullup
           bcf      GLCD_RC1,GLCD_CNTDI          ; aviso o LCD que vou enviar um comando
           bsf      GLCD_RC1,GLCD_CNTRW          ; aviso que o LCD deve me devolver um dado
           movf     GLCD_RC1,w                   ; carrego o buffer de controle em W
           movwf    GLCD_CNT                     ; coloco o controle na porta de controle
           nop                                   ; espera estabilizar
           nop                                   ; espera estabilizar
           nop                                   ; espera estabilizar
GLCD_Wait1 bsf      GLCD_RC1,GLCD_CNTE           ; levanto o sinal de envio no buffer de controle
           movf     GLCD_RC1,w                   ; carrego o buffer de controle em W
           movwf    GLCD_CNT                     ; coloco o controle na porta de controle
           movlw    10                           ; 10us
           call     GLCD_Delay                   ; espera 10us (microssegundos)
           bcf      GLCD_RC1,GLCD_CNTE           ; abaixo o sinal de envio no buffer de controle
           movf     GLCD_RC1,w                   ; carrego o buffer de controle em W
           movwf    GLCD_CNT                     ; coloco o controle na porta de controle
           nop                                   ; espera estabilizar
           nop                                   ; espera estabilizar
           movf     GLCD_DATAIN,w                ; leio do dado que veio naporta de dados
           andlw    BUSY                         ; teste do bit BUSY
           xorlw    BUSY                         ; teste do bit BUSY
           btfss    STATUS,Z                     ; LCD estava ocupado???
           goto     GLCD_Wait1                   ; SIM --> espera desocupar
           bsf      INTCON2,RBPU                 ; NAO --> desabilita resistores pullup
           movlw    0x00                         ; mascara de I/O
           movwf    GLCD_DATATRIS                ; porta de dados como saida
           movwf    GLCD_DATAOUT                 ; limpa a porta de dados
           return                                ; fim 



; ----------------------------------------------------------------------------------------------------------
; Delay em microssegundos
; ENTRADA: W = tempo em microssegundos
; ----------------------------------------------------------------------------------------------------------
GLCD_Delay  movwf   GLCD_Delay1                  ; numero de microssegundos
_delayloop  nop                                  ;  200ns
            nop                                  ;  200ns
            decfsz  GLCD_Delay1, f               ;  200ns
            goto    _delayloop                   ;  400ns
                                                 ; 1000ns = 1us --> total
            return                               ; fim



; ----------------------------------------------------------------------------------------------------------
; Limpa o(s) lado(s) habilitado(s) do LCD
;
; ENTRADA: nao ha
;
; SAIDA  : 1 - LCD LIMPO
;          2 - LCD Posicionado em 0,0
;          3 - Ambos os lados desabilitados
; ----------------------------------------------------------------------------------------------------------
GLCD_Clear movlw  0x00                           ; caracter de preenchimento
           call   GLCD_Fill                      ; vai preencher o LCD 
           movlw  Y_ADDRESS                      ; posiciona coluna 0
           call   GLCD_Command                   ; envia
           movlw  X_ADDRESS                      ; posiciona pagina 0
           call   GLCD_Command                   ; envia
           movlw  START_LINE                     ; posiciona linha 0
           call   GLCD_Command                   ; envia
           return                                ; fim



; ----------------------------------------------------------------------------------------------------------
; Inverte os lados do LCD:
; Se o lado 1 estiver habilitado e o lado 2 desabilitado, entao o lado 1 sera desabilitado
; e o lado 2 sera habilitado ou vice-versa.
;
; ENTRADA: Um dos lados deve estar selecionado e o outro desabilitado antes de chamar esta funcao.
;
; SAIDA  : habilitacao dos lados invertidas
;
; ----------------------------------------------------------------------------------------------------------
GLCD_Swap   btg     GLCD_RC1,GLCD_CNTCS1         ; inverte o lado 1
            btg     GLCD_RC1,GLCD_CNTCS2         ; inverte o lado 2
            movf    GLCD_RC1,w                   ; carrega o buffer de controle
            movwf   GLCD_CNT                     ; envia o controle
            return                               ; fim


; ----------------------------------------------------------------------------------------------------------
; Le um byte na posicao atual do LCD.
;
; ENTRADA: 1 - Um dos lados deve estar habilitado
;          2 - X_ADDRESS deve estar posicionado
;          3 - Y_ADDRESS deve estar posicionado
;
; SAIDA  : W contem o byte lido
;
; ----------------------------------------------------------------------------------------------------------
GLCD_Read  call     GLCD_Wait                    ; espera liberar o LCD
           movlw    0xFF                         ; mascara de entrada
           movwf    GLCD_DATATRIS                ; porta de dados para entrada
           bcf      INTCON2,RBPU                 ; habilita resistores pullup
           bsf      GLCD_RC1,GLCD_CNTDI          ; DI --> data
           movf     GLCD_RC1,w                   ; carrega registrador de controle
           movwf    GLCD_CNT                     ; envia para a porta de controle
           bsf      GLCD_RC1,GLCD_CNTRW          ; RW --> read
           movf     GLCD_RC1,w                   ; carrega registrador de controle
           movwf    GLCD_CNT                     ; envia para a porta de controle
           bsf      GLCD_RC1,GLCD_CNTE           ; E = 1
           movf     GLCD_RC1,w                   ; carrega registrador de controle
           movwf    GLCD_CNT                     ; envia para a porta de controle
           nop                                   ; espera estabilizar
           nop                                   ; espera estabilizar
           nop                                   ; espera estabilizar
           nop                                   ; espera estabilizar
           nop                                   ; espera estabilizar
           movf     GLCD_DATAIN,w                ; le a porta de dados
           movwf    GLCD_W1                      ; salva byte lido
           bcf      GLCD_RC1,GLCD_CNTE           ; E = 0
           movf     GLCD_RC1,w                   ; carrega registrador de controle
           movwf    GLCD_CNT                     ; envia para a porta de controle
           call     GLCD_Wait                    ; aguarda liberar o LCD
           bsf      INTCON2,RBPU                 ; desabilita resistores pullup
           clrf     GLCD_DATATRIS                ; porta de dados para saida
           bcf      GLCD_RC1,GLCD_CNTRW          ; RW --> write
           movf     GLCD_RC1,w                   ; carrega registrador de controle
           movwf    GLCD_CNT                     ; envia para a porta de controle
           movf     GLCD_W1,w                    ; W = dado lido
           return                                ; fim



; ----------------------------------------------------------------------------------------------------------
; Envia um byte ao LCD
;
; ENTRADA: W contem o byte a ser enviado
;
; SAIDA  : cartacter exibido no LCD
; ----------------------------------------------------------------------------------------------------------
GLCD_Write bcf      INTCON2,RBPU                 ; conecta os resistores pull-up do PORTB
           movwf    GLCD_W1                      ; salva o valor a enviar
           call     GLCD_Wait                    ; espera o LCD ficar pronto
           bsf      GLCD_RC1,GLCD_CNTDI          ; especifica modo de dados
           bcf      GLCD_RC1,GLCD_CNTRW          ; especifica gravacao
           movf     GLCD_RC1,w                   ; carrega  buffer de controle
           movwf    GLCD_CNT                     ; envia o buffer de controle
           nop                                   ; espera estabilizar
           nop                                   ; espera estabilizar
           nop                                   ; espera estabilizar
           bsf      GLCD_RC1,GLCD_CNTE           ; levanta o sinal de envio
           movf     GLCD_RC1,w                   ; carrega bufer de controle
           movwf    GLCD_CNT                     ; envia controle
           movlw    10                           ; 16us
           call     GLCD_Delay                   ; mantem o sinal levantado
           movf     GLCD_W1,w                    ; recupera o valor a ser enviado
	   movwf    GLCD_DATAOUT                 ; coloca o valor na porta de dados
           movlw    10                           ; 16us
           call     GLCD_Delay                   ; segura o sinal levandado
           bcf      GLCD_RC1,GLCD_CNTE           ; abaixa o sinal de envio
           movf     GLCD_RC1,w                   ; carrega o buffer de controle
           movwf    GLCD_CNT                     ; envia buffer de controle
           bsf      INTCON2,RBPU                 ; desconecta resistores pull-up de PORTB
           return                                ; fim



; ----------------------------------------------------------------------------------------------------------------
; Preenche um dos lados ou os dois lados do LCD simultaneamente, dependendo de que lado
; esteja habilitado no momento.
;
; ENTRADA: 1 - W deve conter o caracter de preenchimento
;          2 - O lado desejado deve estar selecionado.
;          3 - Ambos os lados podem estar selecionados ao mensmo tempo
;
; SAIDA  : 1 - LCD preenchido com o caracter especificado em W 
;          2 - Os dois lados serao desabilitados
;
; OBS: Esta funcao e usada para para limpar o LCD com W=0x00.
;
; ----------------------------------------------------------------------------------------------------------------
GLCD_FILL
GLCD_Fill   movwf     GLCD_W2                    ; salva caracter de preenchimento
            clrf      GLCD_PAGE                  ; GLCD_W3=0 --> numero da pagina
GLCD_Clear0 movlw     Y_ADDRESS                  ; col=0
            call      GLCD_Command               ; SET Y ADDRESS
            movlw     X_ADDRESS                  ; comando SET X ADDRESS
            iorwf     GLCD_PAGE,w                ; insere o numero da pagina
            call      GLCD_Command               ; SET X ADDRESS
            movlw     START_LINE                 ; comando START LINE
            call      GLCD_Command               ; START LINE
            movlw     .64                        ; numero total de colunas
            movwf     GLCD_COL                   ; carrega contador de colunas
GLCD_Clear1 movf      GLCD_W2,w                  ; recupera caracter de preenchimento
            call      GLCD_Write                 ; envia o byte para linpar
            decfsz    GLCD_COL,f                 ; conta colunas
            goto      GLCD_Clear1                ; continua limpando a pagina
            incf      GLCD_PAGE,f                ; proxima paginas
            btfss     GLCD_PAGE,3                ; verifica se estourou
            goto      GLCD_Clear0                ; continua se nao estourou
            bcf       GLCD_RC1,GLCD_CNTCS1       ; desabilita o lado 1
            bcf       GLCD_RC1,GLCD_CNTCS2       ; desabilita o lado 2
            movf      GLCD_RC1,w                 ; carrega o registrador de controle
            movwf     GLCD_CNT                   ; envia os controles
            return                               ;



; ----------------------------------------------------------------------------------------------------------------
; Habilita Lado 1
; ENTRADA: nao ha
; ----------------------------------------------------------------------------------------------------------------
GLCD_Enable1 bsf       GLCD_RC1,GLCD_CNTCS1      ; habilita o lado 1
             movf      GLCD_RC1,w                ; carrega o registrador de controle
             movwf     GLCD_CNT                  ; envia os controles
             return



; ----------------------------------------------------------------------------------------------------------------
; Habilita Lado 2
; ENTRADA: nao ha
; ----------------------------------------------------------------------------------------------------------------
GLCD_Enable2 bsf       GLCD_RC1,GLCD_CNTCS2      ; habilita o lado 2
             movf      GLCD_RC1,w                ; carrega o registrador de controle
             movwf     GLCD_CNT                  ; envia os controles
             return




; ----------------------------------------------------------------------------------------------------------------
; Desabilita Lado 1
; ENTRADA: nao ha
; ----------------------------------------------------------------------------------------------------------------
GLCD_Disable1 
             bcf       GLCD_RC1,GLCD_CNTCS1      ; desabilita o lado 1
             movf      GLCD_RC1,w                ; carrega o registrador de controle
             movwf     GLCD_CNT                  ; envia os controles
             return


; ----------------------------------------------------------------------------------------------------------------
; Desabilita Lado 2
; ENTRADA: nao ha
; ----------------------------------------------------------------------------------------------------------------
GLCD_Disable2 
             bcf       GLCD_RC1,GLCD_CNTCS2      ; desabilita o lado 2
             movf      GLCD_RC1,w                ; carrega o registrador de controle
             movwf     GLCD_CNT                  ; envia os controles
             return
